<?
// some handy string functions
class String
{
	var $value;

	// set defaults
	function String($s)
	{
		$this->value = $s;
		$this->startGroupToken = "(";
		$this->endGroupToken = ")";
	}

	// add group tokens, usually to put something in brackets
	// e.g. (BL)2
	// but not (B)2 -- this becomes B2
	function addGroup()
	{
		if (strlen($this->value)>1)
			return $this->startGroupToken . $this->value .  $this->endGroupToken;
		else
			return $this->value;
	}

	// find index of a matching group token (e.g. ending bracket)
	// note: will fail if the token is missing.
	function findMatch($startToken="(", $endToken=")")
	{
			$i=0;
			do {
				$c = substr($this->value, $i, 1);
				if ($c==$startToken)
					$open++;
				if ($c==$endToken)
					$open--;
				$i++;
			} while ($open>0);
			return $i-1;
	}
}

// a class to parse, process, generate and print algorithms
class Alg
{
	// transform an algorithm from string to nested arrays
	function parse($s)
	{
		$c = substr($s, 0, 1);
		// grouped alg, starts at a open bracket
		 if ($c=='(')
		 {
			$string = new String($s);
			$endtoken = $string->findMatch();
			// parse the sub alg recursively
			$subAlg =  array(Alg::parse(substr($s,1, $endtoken-1)));
			$subAlg[0]['count'] = substr($s, $endtoken+1, 1);
			if ($endtoken < strlen($s)-2)
			{
				// parse the rest recursively
				$endAlg = Alg::parse(substr($s, $endtoken+2));
				return array_merge($subAlg, $endAlg);
			}
			else
				return $subAlg;
		}
		// non-grouped, simple algs
		$out = array();
		if (strlen($s)>1)
		{
			$q = substr($s, 1, 1);
			if (is_numeric($q) || $q==chr(39))
				// maybe this can be formulated more simplistic
				return array_merge(array(array($c, "count"=>$q)), Alg::parse(substr($s, 2)));
			else
				return array_merge(array($c), Alg::parse(substr($s, 1)));
		}
		else
			return array($s);
	}

	// once parsed, transform back to string
	// alg is a nested array of operators and 'count' values
	function toString($alg)
	{
		$out = '';
		foreach ($alg as $op)
		{
			if (!is_numeric($op) && $op!=chr(39))
				if (is_array($op))
				{
					// recursively convert grouped algs (which are also nested arrays)
					$string = new String(Alg::toString($op));
					$out .= $string->addGroup();
					if ($op['count']>1 && count($op[0])>1)
						$out .= "*";
					$out .= $op['count'];
				}
				else
					$out .= $op[0];
		}
		return $out;
	}

	// generate a random algorithm
	function randomAlg($minLength, $maxLength)
	{
		$ops = array('R','L','U','D','B','F',  'r', 'l', 'u', 'd', 'b', 'f',  'M', 'E', 'S',  'x', 'y', 'z');
		$qs = array("","2",chr(39));
		$cs = array(2,3,4,5);
		$n = rand($minLength, $maxLength);
		$out = '';
		foreach (range(1,$n) as $op)
		{
			$randOp = $ops[rand(0,count($ops)-1)];
			$randQ = $qs[rand(0,count($qs)-1)];
			$out .= $randOp . $randQ;
			//echo rand()/getrandmax() ."\n";
			if (rand()/getrandmax() < 1/$maxLength)
			{
				$randC = $cs[rand(0,count($cs)-1)];
				// recursively add a grouped alg
				$s = new String(Alg::randomAlg($minLength, $maxLength));
				$out .= $s->addGroup() . $randC;
			}
		}
		return $out;
	}

	// test if s->parse->toString equals s
	function testParser($numTests, $minLength, $maxLength)
	{
		foreach (range(1,$numTests) as $n)
		{
			$alg = Alg::randomAlg($minLength, $maxLength);
			$p = Alg::parse($alg);
			$s = Alg::toString($p);
			if ($s==$alg)
				echo "success";
			else
				echo "fail";
			echo " : " . $s . "\n";
		}
	}
}

Alg::testParser(100, 20, 30);
?>